/* <sys/stat.h>

   Written by Klaus K"ampf, kkaempf@progis.de

 */
#ifndef _SYS_STAT_H
#define _SYS_STAT_H

#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif

# ifdef __cplusplus
extern "C" {
# endif

#define _STAT_T
#define _STAT_T_

struct stat {
	dev_t	st_dev;
	ino_t	st_ino[3];	/* bogus array, compatible with VAXC */
	unsigned short st_mode;
	int	st_nlink;
	unsigned st_uid;
	unsigned short st_gid;
	dev_t	st_rdev;
	off_t	st_size;
	unsigned st_atime;
	unsigned st_mtime;
	unsigned st_ctime;
	char	st_fab_rfm;
	char	st_fab_rat;
	char	st_fab_fsz;
	unsigned st_fab_mrs;
} __attribute__ ((packed));

typedef struct stat stat_t;

#define S_IFMT	 0170000 /* type of file */
#define	S_IFDIR	 0040000 /* directory */
#define	S_IFCHR  0020000 /* character special */
#define	S_IFBLK	 0060000 /* block special */
#define	S_IFREG  0100000 /* regular */
#define	S_IFMPC  0030000 /* multiplexed char special */
#define	S_IFMPB  0070000 /* multiplexed block special */
#define S_ISUID  0004000 /* set user id on execution */
#define S_ISGID  0002000 /* set group id on execution */
#define S_ISVTX  0001000 /* save swapped text even after use */
#define S_IREAD  0000400 /* read permission, owner */
#define S_IWRITE 0000200 /* write permission, owner */
#define S_IEXEC  0000100 /* execute/search permission, owner */

#define S_IRWXU  0000700    /* read,write,execute perm: owner */
#define S_IRUSR  0000400    /* read permission: owner */
#define S_IWUSR  0000200    /* write permission: owner */
#define S_IXUSR  0000100    /* execute/search permission: owner */
#define S_IRWXG  0000070    /* read,write,execute perm: group */
#define S_IRGRP  0000040    /* read permission: group */
#define S_IWGRP  0000020    /* write permission: group */
#define S_IXGRP  0000010    /* execute/search permission: group */
#define S_IRWXO  0000007    /* read,write,execute perm: other */
#define S_IROTH  0000004    /* read permission: other */
#define S_IWOTH  0000002    /* write permission: other */
#define S_IXOTH  0000001    /* execute/search permission: other */



#ifdef __STDC__
int  fstat(int fileno, stat_t *buf);
int  stat(const char *name, stat_t *buf);
#else	/*!__STDC__ => -traditional */
int  fstat(), stat();
#endif	/*?__STDC__*/

#ifndef S_ISDIR
#define S_ISDIR(mode)	(((mode) & S_IFMT) == S_IFDIR)
#endif
#ifndef S_ISCHR
#define S_ISCHR(mode)	(((mode) & S_IFMT) == S_IFCHR)
#endif
#ifndef S_ISBLK
#define S_ISBLK(mode)	(((mode) & S_IFMT) == S_IFBLK)
#endif
#ifndef S_ISREG
#define S_ISREG(mode)	(((mode) & S_IFMT) == S_IFREG)
#endif

# ifdef __cplusplus
}
# endif
#endif	/*_SYS_STAT_H*/
