/* <dmpdef.h>
 *
 *	System dump file.
 */
#ifndef _DMPDEF_H
#define _DMPDEF_H

/*
   Layout of the header block of the system dump file
   (which is the first disk block of SYS$SYSTEM:SYSDUMP.DMP).
 */
#define DMP$V_OLDDUMP	0
#define DMP$V_EMPTY	1
#define DMP$V_HDRBLK	2
#define DMP$V_ERRLOG	3
#define DMP$V_MEMORY	4
#define DMP$V_SPT	5
#define DMP$V_WRITECOMP 6
#define DMP$V_ERRLOGCOMP 7
#define DMP$M_OLDDUMP	(1<<DMP$V_OLDDUMP)	/* 0x01 */
#define DMP$M_EMPTY	(1<<DMP$V_EMPTY)	/* 0x02 */
#define DMP$M_HDRBLK	(1<<DMP$V_HDRBLK)	/* 0x04 */
#define DMP$M_ERRLOG	(1<<DMP$V_ERRLOG)	/* 0x08 */
#define DMP$M_MEMORY	(1<<DMP$V_MEMORY)	/* 0x10 */
#define DMP$M_SPT	(1<<DMP$V_SPT)		/* 0x20 */
#define DMP$M_WRITECOMP (1<<DMP$V_WRITECOMP)	/* 0x40 */
#define DMP$M_ERRLOGCOMP (1<<DMP$V_ERRLOGCOMP)	/* 0x80 */
#define DMP$V_DUMP_STYLE 8
#define DMP$M_DUMP_STYLE 0x0F00
#define DMP$S_DUMP_STYLE 4
#define DMP$V_BITS_12_15 12
#define DMP$M_BITS_12_15 0xF000
#define DMP$S_BITS_12_15 4
#define DMP$V_BITS_16_31 16
#define DMP$M_BITS_16_31 0xFFFF0000
#define DMP$V_BITS_16_31 16
#define DMP$K_FULL_DUMP   0		/* full physical memory dump (old way) */
#define DMP$K_SUBSET_DUMP 1		/* subsetted dump file (not all info saved) */
#define DMP$C_DUMP_LUMP 127		/* number of SPTEs required for subset dumps */
#define DMP$C_MEMDSCSIZ 8		/* size of one memory descriptor */
#define DMP$C_NMEMDSC	8		/* number of memory descriptors */
#define DMP$K_DUMPVER	1328		/* dump file version number */
#define DMP$K_LENGTH	132		/* length of file header */
#define DMP$C_LENGTH	132
struct dmpdef {
    unsigned long dmp$l_errseq;		/* last error log seq. number */
    union {
	struct {
	    unsigned short dmp$w_flags; /* dump file flags */
	    unsigned	: 8;		/* unsigned char fill; unused */
	    unsigned char dmp$b_erlbufpages; /* pages per erl buffer; if zero, correct value is one */
	} dmp$r_flags_fields;
	struct {	/* dump_style bitfield settings */
	    unsigned dmp$v_olddump : 1; /* set if dump already analyzed */
	    unsigned dmp$v_empty   : 1; /* set if dump has no data blocks */
	    unsigned dmp$v_hdrblk  : 1; /* set if error writing first block of dump header */
	    unsigned dmp$v_errlog  : 1; /* set if error writing blocks 2-3 of dump header */
	    unsigned dmp$v_memory  : 1; /* set if error writing memory contents to dump file */
	    unsigned dmp$v_spt	   : 1; /* set if error writing system page table blocks to dump file */
	    unsigned dmp$v_writecomp  : 1; /* set if dump write was completed */
	    unsigned dmp$v_errlogcomp : 1; /* set if header/error logs write was completed */
	    unsigned dmp$v_dump_style : 4; /* set to differentiate between different style dumps */
	    unsigned dmp$v_bits_12_15 : 4; /* unused */
	    unsigned dmp$v_bits_16_31 : 16; /* overlays dmp$b_bufpages */
	} dmp$r_flags_bits;
    } dmp$r_flags_overlay;
    unsigned long dmp$l_sbr;		/* system base register */
    unsigned long dmp$l_slr;		/* system length register */
    unsigned long dmp$l_ksp;		/* kernel stack pointer */
    unsigned long dmp$l_esp;		/* executive stack pointer */
    unsigned long dmp$l_ssp;		/* supervisor stack pointer */
    unsigned long dmp$l_usp;		/* user stack pointer */
    unsigned long dmp$l_isp;		/* interrupt stack pointer */
    union {
	unsigned long dmp$l_memory_desc_space[16]; /* reserve space for 8 memory descriptors (2 longwords each) */
	union {
	    unsigned long dmp$l_memdsc; /* 8 memory descriptors */
	    struct {
		unsigned dmp$v_pagcnt  : 24;	/* number of pages in memory */
		unsigned dmp$v_tr      :  8;	/* TR number for memory */
		unsigned dmp$v_basepfn : 32;	/* base PFN for memory */
	    } dmp$r_memdsc_bits;
	} dmp$r_memdsc_overlay;
    } dmp$r_memory_desc;
    unsigned long dmp$l_sysver;		/* system version number */
    unsigned long dmp$l_check;		/* ones complement of sysver */
    unsigned short dmp$w_dumpver;	/* dump file version number */
    unsigned short dmp$w_erlbufcnt;	/* number of error log buffers */
    unsigned short dmp$w_erlbufhead;	/* ring buffer head index */
    unsigned short dmp$w_erlbuftail;	/* ring buffer tail index */
    unsigned long dmp$l_errstatus;	/* last error status from dump write */
    unsigned long dmp$l_dumperrs;	/* count of errors during dump write */
    unsigned long dmp$l_mempagcnt;	/* count of memory pages dumped */
    unsigned long dmp$l_saveprccnt;	/* count of saved processes (subset dump) */
    unsigned long dmp$l_crasherl;	/* system crash err log entry */
};

#endif	/*_DMPDEF_H*/
