/* <ensdef.h>
 *
 *	Generalized Event Notification definitions
 */
#ifndef _ENSDEF_H
#define _ENSDEF_H

/*
   This module contains definitions for all the flags used
   in all the event notification system services.  It also
   defines the layout of the event server's base page.
 */
#define ENS$V_ACB_IN_USE   0
#define ENS$V_EVENT_LOGGED 1
#define ENS$M_ACB_IN_USE   (1<<ENS$V_ACB_IN_USE)	/* 0x01 */
#define ENS$M_EVENT_LOGGED (1<<ENS$V_EVENT_LOGGED)	/* 0x02 */
#define ENS$V_LOCAL	0
#define ENS$V_DEVICE	1
#define ENS$V_IMAGE	2
#define ENS$V_FLOST	3
#define ENS$V_ULOST	4
#define ENS$V_WILDCARD	5
#define ENS$M_LOCAL	(1<<ENS$V_LOCAL)	/* 0x01 */
#define ENS$M_DEVICE	(1<<ENS$V_DEVICE)	/* 0x02 */
#define ENS$M_IMAGE	(1<<ENS$V_IMAGE)	/* 0x04 */
#define ENS$M_FLOST	(1<<ENS$V_FLOST)	/* 0x08 */
#define ENS$M_ULOST	(1<<ENS$V_ULOST)	/* 0x10 */
#define ENS$M_WILDCARD	(1<<ENS$V_WILDCARD)	/* 0x20 */
#define ENS$V_EXAMINE	0
#define ENS$V_RECEIVE	1
#define ENS$V_DECLARE	2
#define ENS$V_DELETE	3
#define ENS$V_CONTROL	4
#define ENS$M_EXAMINE	(1<<ENS$V_EXAMINE)	/* 0x01 */
#define ENS$M_RECEIVE	(1<<ENS$V_RECEIVE)	/* 0x02 */
#define ENS$M_DECLARE	(1<<ENS$V_DECLARE)	/* 0x04 */
#define ENS$M_DELETE	(1<<ENS$V_DELETE)	/* 0x08 */
#define ENS$M_CONTROL	(1<<ENS$V_CONTROL)	/* 0x10 */
#define ENS$K_FKB_START 48		/** start of FKB */
#define ENS$C_FKB_LENGTH 24
#define ENS$K_ACB_START 72		/** start of ACB */
#define ENS$C_ACB_LENGTH 28
/* End of quadword aligned structures */
#define ENS$C_MAX_CUST_EVENTS 64
#define ENS$C_MAX_EVENTS 128
#define ENS$K_LENGTH	512		/* length of block */
#define ENS$C_LENGTH	512
struct ensdef {
    unsigned long ens$l_epbqfl;		/* non-paged work queue forward link */
    unsigned long ens$l_epbqbl;		/* non-paged work queue backward link */
    unsigned short ens$w_size;		/* structure size in bytes */
    unsigned char ens$b_type;		/* structure type code */
    unsigned char ens$b_subtype;	/* structure subtype code */
    union {
	unsigned long ens$l_flags;	/* miscellaneous flags */
	struct {			/* flags used in base block */
	    unsigned ens$v_acb_in_use	: 1; /* ACB/fork block in use */
	    unsigned ens$v_event_logged : 1; /* an event has been logged */
	    unsigned			: 6;
	} ens$r_flag_bits;
	struct {			/* flags common to all services */
	    unsigned ens$v_local   : 1; /* local node */
	    unsigned ens$v_device  : 1; /* device related */
	    unsigned ens$v_image   : 1; /* cancel on image exit */
	    unsigned ens$v_flost   : 1; /* fork lost */
	    unsigned ens$v_ulost   : 1; /* user lost */
	    unsigned ens$v_wildcard: 1; /* wildcarded event name */
	    unsigned		   : 2;
	} ens$r_common_flag_bits;
	struct {			/* access mask bits */
	    unsigned ens$v_examine : 1; /* identifier has examine access */
	    unsigned ens$v_receive : 1; /* identifier has receive access */
	    unsigned ens$v_declare : 1; /* identifier has declare access */
	    unsigned ens$v_delete  : 1; /* identifier has delete access */
	    unsigned ens$v_control : 1; /* identifier has control access */
	    unsigned		   : 3;
	} ens$r_access_bit_names;
    } ens$r_flags_overlay;
    int ens$q_log_time[2][2];		/* time first log counter was incremented */
 /* start of quadword aligned structures */
    unsigned long ens$l_ensqfl;		/* paged work queue forward link */
    unsigned long ens$l_ensqbl;		/* paged work queue backward link */
    unsigned long ens$l_efdqfl;		/* orb queue forward link */
    unsigned long ens$l_efdqbl;		/* orb queue backward link */
    unsigned long ens$l_fqfl;		/* fork queue forward link */
    unsigned long ens$l_fqbl;		/* fork queue backward link */
    unsigned short ens$w_fkb_size;	/* size of FKB in bytes */
    unsigned char ens$b_fkb_type;	/* structure type of FKB */
    union {
	unsigned char ens$b_flck;	/* fork lock number */
	unsigned char ens$b_fipl;	/* fork IPL */
    } ens$r_flck_overlay;
    unsigned long ens$l_fpc;		/* fork PC */
    unsigned long ens$l_fr3;		/* fork R3 */
    unsigned long ens$l_fr4;		/* fork R4 */
    unsigned long ens$l_astqfl;		/* AST queue forward link */
    unsigned long ens$l_astqbl;		/* AST queue backward link */
    unsigned short ens$w_acb_size;	/* structure size in bytes */
    unsigned char ens$b_acb_type;	/* structure type code */
    unsigned char ens$b_rmod;		/* request access mode */
    unsigned long ens$l_pid;		/* server process id */
    unsigned long ens$l_ast;		/* AST routine address */
    unsigned long ens$l_astprm;		/* AST parameter */
    unsigned long ens$l_kast;		/* internal kernel mode xfer address */
    char ensdef_fill[28];
    short ens$w_cust_counts[64];	/* customer lost event counters */
    short ens$w_vms_counts[128];	/* VMS lost event counters */
};

#endif	/*_ENSDEF_H*/
